@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2012, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2014, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,31 +20,21 @@
  *
  ***************************************************************************/
 
-/* Now include the setup.h file from libcurl's private libdir (the source
+/* Now include the curl_setup.h file from libcurl's private libdir (the source
    version, but that might include "curl_config.h" from the build dir so we
    need both of them in the include path), so that we get good in-depth
    knowledge about the system we're building this on */
 
 #define CURL_NO_OLDIES
 
-#include "setup.h"
+#include "curl_setup.h"
 
 #include <curl/curl.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
 
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
-#endif
 #ifdef HAVE_SYS_SELECT_H
 /* since so many tests use select(), we can just as well include it here */
 #include <sys/select.h>
 #endif
-#ifdef HAVE_UNISTD_H
-/* at least somewhat oldish FreeBSD systems need this for select() */
-#include <unistd.h>
-#endif
 
 #ifdef TPF
 #  include "select.h"
@@ -68,6 +58,8 @@ extern struct timeval tv_test_start; /* for test timing */
 extern int select_wrapper(int nfds, fd_set *rd, fd_set *wr, fd_set *exc,
                           struct timeval *tv);
 
+extern void wait_ms(int ms); /* wait this many milliseconds */
+
 extern int test(char *URL); /* the actual test function provided by each
                                individual libXXX.c file */
 
